@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*
 from __future__ import unicode_literals
-
-from warnings import warn
+from __future__ import absolute_import
 
 from django.db import models
 from django.contrib.sites import managers
@@ -82,22 +81,7 @@ class SpanningCurrentSiteManager(managers.CurrentSiteManager):
     def _get_related_model(self, model, fieldname):
         """Given a model and the name of a ForeignKey or ManyToManyField column
         as a string, returns the associated model."""
-        return model._meta.get_field_by_name(fieldname)[0].rel.to
-
-
-class PathAssistedCurrentSiteManager(models.CurrentSiteManager):
-    """
-    Deprecated: Use multisite.managers.SpanningCurrentSiteManager instead.
-    """
-    def __init__(self, field_path):
-        warn(('Use multisite.managers.SpanningCurrentSiteManager instead of '
-              'multisite.managers.PathAssistedCurrentSiteManager'),
-             DeprecationWarning, stacklevel=2)
-        super(PathAssistedCurrentSiteManager, self).__init__()
-        self.__field_path = field_path
-
-    def get_queryset(self):
-        from django.contrib.sites.models import Site
-        return super(models.CurrentSiteManager, self).get_queryset().filter(
-                    **{self.__field_path: Site.objects.get_current()}
-                )
+        try:
+            return model._meta.get_field(fieldname).remote_field.model
+        except AttributeError:
+            return model._meta.get_field(fieldname).rel.to
