@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
+from __future__ import absolute_import
 
 import os
 import tempfile
@@ -8,36 +9,39 @@ try:
 except ImportError:
     from urllib.parse import urlsplit, urlunsplit
 
+import django
 from django.conf import settings
 from django.contrib.sites.models import Site, SITE_CACHE
+from django.core.exceptions import DisallowedHost
 from django.core import mail
 
+from django.core.cache import caches
+
 try:
-    from django.core.cache import caches
+    # Django > 1.10 uses MiddlewareMixin
+    from django.utils.deprecation import MiddlewareMixin
 except ImportError:
-    # Django < 1.7 compatibility
-    from django.core.cache import get_cache
-else:
-    def get_cache(cache_alias):
-        return caches[cache_alias]
+    MiddlewareMixin = object
 
 from django.core.exceptions import ImproperlyConfigured
-from django.core.urlresolvers import get_callable
+
+try:
+    from django.urls import get_callable
+except ImportError:
+    # Django < 1.10 compatibility
+    from django.core.urlresolvers import get_callable
+
 from django.db.models.signals import pre_save, post_delete, post_init
 from django.http import Http404, HttpResponsePermanentRedirect
 
-try:
-    # Deprecated in Django 1.5
-    from django.utils.hashcompat import md5_constructor
-except ImportError:
-    # The above has been removed in Django 1.6
-    from hashlib import md5 as md5_constructor
+from hashlib import md5 as md5_constructor
 
 from .models import Alias
 
 
-class DynamicSiteMiddleware(object):
-    def __init__(self):
+class DynamicSiteMiddleware(MiddlewareMixin):
+    def __init__(self, *args, **kwargs):
+        super(DynamicSiteMiddleware, self).__init__(*args, **kwargs)
         if not hasattr(settings.SITE_ID, 'set'):
             raise TypeError('Invalid type for settings.SITE_ID: %s' %
                             type(settings.SITE_ID).__name__)
@@ -50,7 +54,7 @@ class DynamicSiteMiddleware(object):
             settings.CACHES[self.cache_alias].get('KEY_PREFIX', '')
         )
 
-        self.cache = get_cache(self.cache_alias)
+        self.cache = caches[self.cache_alias]
         post_init.connect(self.site_domain_cache_hook, sender=Site,
                           dispatch_uid='multisite_post_init')
         pre_save.connect(self.site_domain_changed_hook, sender=Site)
@@ -136,8 +140,17 @@ class DynamicSiteMiddleware(object):
         if callable(fallback):
             view = fallback
         else:
-            view = get_callable(fallback, can_fail=True)
-            if not callable(view):
+            try:
+                view = get_callable(fallback)
+                if django.VERSION < (1,8):
+                    # older django's get_callable falls through on error,
+                    # returning the input as output
+                    # which notably is definitely not a callable here
+                    if not callable(view):
+                        raise ImportError()
+            except ImportError:
+                # newer django forces this to be an error, which is tidier.
+                # we rewrite the error to be a bit more helpful to our users.
                 raise ImproperlyConfigured(
                     'settings.MULTISITE_FALLBACK is not callable: %s' %
                     fallback
@@ -160,7 +173,12 @@ class DynamicSiteMiddleware(object):
         return HttpResponsePermanentRedirect(url)
 
     def process_request(self, request):
-        netloc = request.get_host().lower()
+        try:
+            netloc = request.get_host().lower()
+        except DisallowedHost:
+            settings.SITE_ID.reset()
+            return self.fallback_view(request)
+
         cache_key = self.get_cache_key(netloc)
 
         # Find the Alias in the cache
@@ -203,8 +221,9 @@ class DynamicSiteMiddleware(object):
         self.cache.clear()
 
 
-class CookieDomainMiddleware(object):
-    def __init__(self):
+class CookieDomainMiddleware(MiddlewareMixin):
+    def __init__(self, *args, **kwargs):
+        super(CookieDomainMiddleware, self).__init__(*args, **kwargs)
         self.depth = int(getattr(settings, 'MULTISITE_COOKIE_DOMAIN_DEPTH', 0))
         if self.depth < 0:
             raise ValueError(
