@@ -1,4 +1,5 @@
 from __future__ import unicode_literals
+from __future__ import absolute_import
 
 import operator
 from functools import reduce
@@ -9,11 +10,8 @@ from django.core.validators import validate_ipv4_address
 from django.db import connections, models, router
 from django.db.models import Q
 from django.db.models.signals import pre_save, post_save
-try:
-    from django.db.models.signals import post_migrate
-except ImportError:
-    # Django < 1.7 compatibility
-    from django.db.models.signals import post_syncdb as post_migrate
+from django.db.models.signals import post_migrate
+from django.utils.encoding import python_2_unicode_compatible
 from django.utils.translation import ugettext_lazy as _
 
 from .hacks import use_framework_for_site_cache
@@ -124,7 +122,10 @@ class CanonicalAliasManager(models.Manager):
     def sync_missing(self):
         """Create missing canonical Alias objects based on Site.domain."""
         aliases = self.get_queryset()
-        sites = self.model._meta.get_field('site').rel.to
+        try:
+            sites = self.model._meta.get_field('site').remote_field.model
+        except AttributeError:
+            sites = self.model._meta.get_field('site').rel.to
         for site in sites.objects.exclude(aliases__in=aliases):
             Alias.sync(site=site)
 
@@ -148,6 +149,7 @@ def validate_true_or_none(value):
         raise ValidationError(u'%r must be True or None' % value)
 
 
+@python_2_unicode_compatible
 class Alias(models.Model):
     """
     Model for domain-name aliases for Site objects.
@@ -163,7 +165,9 @@ class Alias(models.Model):
         unique=True,
         help_text=_('Either "domain" or "domain:port"'),
     )
-    site = models.ForeignKey(Site, related_name='aliases')
+    site = models.ForeignKey(
+        Site, related_name='aliases', on_delete=models.CASCADE
+    )
     is_canonical = models.NullBooleanField(
         _('is canonical?'),
         default=None, editable=False,
@@ -184,9 +188,12 @@ class Alias(models.Model):
         unique_together = [('is_canonical', 'site')]
         verbose_name_plural = _('aliases')
 
-    def __unicode__(self):
+    def __str__(self):
         return "%s -> %s" % (self.domain, self.site.domain)
 
+    def __repr__(self):
+        return '<Alias: %s>' % str(self)
+
     def save_base(self, *args, **kwargs):
         self.full_clean()
         # For canonical Alias, domains must match Site domains.
@@ -309,14 +316,7 @@ class Alias(models.Model):
     @classmethod
     def db_table_created_hook(cls, *args, **kwargs):
         """Syncs canonical Alias objects for all existing Site objects."""
-        if kwargs.get('created_models'):
-            # For post_syncdb support in Django < 1.7:
-            # As before, only sync_all if Alias was in
-            # the list of created models
-            if cls in kwargs['created_models']:
-                Alias.canonical.sync_all()
-        else:
-            Alias.canonical.sync_all()
+        Alias.canonical.sync_all()
 
 
 # Hooks to handle Site objects being created or changed
