@@ -1,31 +1,49 @@
+"""
+Tests for django-multisite.
+
+To run this, use:
+$ python -m multisite.tests
+or
+$ python setup.py test
+from the parent directory.
+
+This file uses relative imports and so cannot be run standalone.
+"""
+
 from __future__ import unicode_literals
-
-import os
-import tempfile
-from unittest import skipUnless, skipIf
-import warnings
-
+from __future__ import absolute_import
 
 import django
-from django.conf import settings
-from django.contrib.sites.models import Site
-from django.core.exceptions import (ImproperlyConfigured, SuspiciousOperation,
-                                    ValidationError)
-from django.http import Http404, HttpResponse
-from django.test import TestCase
-from django.test.client import RequestFactory as DjangoRequestFactory
-
-from hacks import use_framework_for_site_cache
+import logging
+import os
+import pytest
+import sys
+import tempfile
+import warnings
+from unittest import skipUnless
 
 try:
-    from django.test.utils import override_settings
+    from unittest import mock
 except ImportError:
-    from override_settings import override_settings
+    import mock
 
-from . import SiteDomain, SiteID, threadlocals
+from django.conf import settings
+from django.conf.urls import url
+from django.contrib.sites.models import Site
+from django.core.exceptions import ImproperlyConfigured, ValidationError
+from django.core.management import call_command
+from django.http import Http404, HttpResponse
+from django.template.loader import get_template
+from django.test import TestCase, override_settings
+from django.test.client import RequestFactory as DjangoRequestFactory
+from django.utils.six import StringIO
+
+from multisite import SiteDomain, SiteID, threadlocals
+
+from .hacks import use_framework_for_site_cache
+from .hosts import ALLOWED_HOSTS, AllowedHosts, IterableLazyObject
 from .middleware import CookieDomainMiddleware, DynamicSiteMiddleware
 from .models import Alias
-from .threadlocals import SiteIDHook
 
 
 class RequestFactory(DjangoRequestFactory):
@@ -39,7 +57,7 @@ class RequestFactory(DjangoRequestFactory):
         return super(RequestFactory, self).get(path=path, data=data,
                                                HTTP_HOST=host, **extra)
 
-
+@pytest.mark.django_db
 @skipUnless(Site._meta.installed,
             'django.contrib.sites is not in settings.INSTALLED_APPS')
 @override_settings(
@@ -57,16 +75,27 @@ class TestContribSite(TestCase):
         self.assertEqual(current_site, self.site)
         self.assertEqual(current_site.id, settings.SITE_ID)
 
+# Because we are a middleware package, we have no views available to test with easily
+# So create one:
+# (This is only used by test_integration)
+urlpatterns = [
+    url(r'^domain/$', lambda request, *args, **kwargs: HttpResponse(str(Site.objects.get_current())))
+]
 
+@pytest.mark.django_db
 @skipUnless(Site._meta.installed,
             'django.contrib.sites is not in settings.INSTALLED_APPS')
 @override_settings(
+    ALLOWED_SITES=['*'],
+    ROOT_URLCONF=__name__, #this means that urlpatterns above is used when .get() is called below.
     SITE_ID=SiteID(default=0),
     CACHE_MULTISITE_ALIAS='multisite',
     CACHES={
+        'default': {'BACKEND': 'django.core.cache.backends.dummy.DummyCache'},
         'multisite': {'BACKEND': 'django.core.cache.backends.dummy.DummyCache'}
     },
     MULTISITE_FALLBACK=None,
+    ALLOWED_HOSTS=ALLOWED_HOSTS
 )
 class DynamicSiteMiddlewareTest(TestCase):
     def setUp(self):
@@ -75,78 +104,79 @@ class DynamicSiteMiddlewareTest(TestCase):
 
         Site.objects.all().delete()
         self.site = Site.objects.create(domain=self.host)
-
-        self.middleware = DynamicSiteMiddleware()
-
-    def tearDown(self):
-        settings.SITE_ID.reset()
+        self.site2 = Site.objects.create(domain='anothersite.example')
 
     def test_valid_domain(self):
         # Make the request
         request = self.factory.get('/')
-        self.assertEqual(self.middleware.process_request(request), None)
+        self.assertEqual(DynamicSiteMiddleware().process_request(request), None)
         self.assertEqual(settings.SITE_ID, self.site.pk)
         # Request again
-        self.assertEqual(self.middleware.process_request(request), None)
+        self.assertEqual(DynamicSiteMiddleware().process_request(request), None)
         self.assertEqual(settings.SITE_ID, self.site.pk)
 
     def test_valid_domain_port(self):
         # Make the request with a specific port
         request = self.factory.get('/', host=self.host + ':8000')
-        self.assertEqual(self.middleware.process_request(request), None)
+        self.assertEqual(DynamicSiteMiddleware().process_request(request), None)
         self.assertEqual(settings.SITE_ID, self.site.pk)
         # Request again
-        self.assertEqual(self.middleware.process_request(request), None)
+        self.assertEqual(DynamicSiteMiddleware().process_request(request), None)
         self.assertEqual(settings.SITE_ID, self.site.pk)
 
     def test_case_sensitivity(self):
         # Make the request in all uppercase
         request = self.factory.get('/', host=self.host.upper())
-        self.assertEqual(self.middleware.process_request(request), None)
+        self.assertEqual(DynamicSiteMiddleware().process_request(request), None)
         self.assertEqual(settings.SITE_ID, self.site.pk)
 
     def test_change_domain(self):
         # Make the request
         request = self.factory.get('/')
-        self.assertEqual(self.middleware.process_request(request), None)
+        self.assertEqual(DynamicSiteMiddleware().process_request(request), None)
         self.assertEqual(settings.SITE_ID, self.site.pk)
         # Another request with a different site
-        site2 = Site.objects.create(domain='anothersite.example')
-        request = self.factory.get('/', host=site2.domain)
-        self.assertEqual(self.middleware.process_request(request), None)
-        self.assertEqual(settings.SITE_ID, site2.pk)
+        request = self.factory.get('/', host=self.site2.domain)
+        self.assertEqual(DynamicSiteMiddleware().process_request(request), None)
+        self.assertEqual(settings.SITE_ID, self.site2.pk)
 
     def test_unknown_host(self):
         # Unknown host
         request = self.factory.get('/', host='unknown')
-        self.assertRaises(Http404,
-                          self.middleware.process_request, request)
+        with self.assertRaises(Http404):
+            DynamicSiteMiddleware().process_request(request)
+        # The middleware resets SiteID to its default value, as given above, on error.
         self.assertEqual(settings.SITE_ID, 0)
+
+    def test_unknown_hostport(self):
         # Unknown host:port
         request = self.factory.get('/', host='unknown:8000')
-        self.assertRaises(Http404,
-                          self.middleware.process_request, request)
+        with self.assertRaises(Http404):
+            DynamicSiteMiddleware().process_request(request)
+        # The middleware resets SiteID to its default value, as given above, on error.
         self.assertEqual(settings.SITE_ID, 0)
 
     def test_invalid_host(self):
         # Invalid host
         request = self.factory.get('/', host='')
-        self.assertRaises(SuspiciousOperation,
-                          self.middleware.process_request, request)
-        self.assertEqual(settings.SITE_ID, 0)
+        with self.assertRaises(Http404):
+            DynamicSiteMiddleware().process_request(request)
+
+    def test_invalid_hostport(self):
         # Invalid host:port
         request = self.factory.get('/', host=':8000')
-        self.assertRaises(SuspiciousOperation,
-                          self.middleware.process_request, request)
-        self.assertEqual(settings.SITE_ID, 0)
+        with self.assertRaises(Http404):
+            DynamicSiteMiddleware().process_request(request)
 
     def test_no_sites(self):
+        # FIXME: this needs to go into its own TestCase since it requires modifying the fixture to work properly
         # Remove all Sites
         Site.objects.all().delete()
         # Make the request
         request = self.factory.get('/')
         self.assertRaises(Http404,
-                          self.middleware.process_request, request)
+                          DynamicSiteMiddleware().process_request, request)
+        # The middleware resets SiteID to its default value, as given above, on error.
         self.assertEqual(settings.SITE_ID, 0)
 
     def test_redirect(self):
@@ -155,7 +185,7 @@ class DynamicSiteMiddlewareTest(TestCase):
         self.assertTrue(alias.redirect_to_canonical)
         # Make the request
         request = self.factory.get('/path', host=host)
-        response = self.middleware.process_request(request)
+        response = DynamicSiteMiddleware().process_request(request)
         self.assertEqual(response.status_code, 301)
         self.assertEqual(response['Location'],
                          "http://%s/path" % self.host)
@@ -166,10 +196,25 @@ class DynamicSiteMiddlewareTest(TestCase):
                              redirect_to_canonical=False)
         # Make the request
         request = self.factory.get('/path', host=host)
-        self.assertEqual(self.middleware.process_request(request), None)
+        self.assertEqual(DynamicSiteMiddleware().process_request(request), None)
         self.assertEqual(settings.SITE_ID, self.site.pk)
 
+    def test_integration(self):
+        """
+        Test that the middleware loads and runs properly under settings.MIDDLEWARE.
+        """
+        resp = self.client.get('/domain/', HTTP_HOST=self.host)
+        self.assertEqual(resp.status_code, 200)
+        self.assertContains(resp, self.site.domain)
+        self.assertEqual(settings.SITE_ID, self.site.pk)
 
+        resp = self.client.get('/domain/', HTTP_HOST=self.site2.domain)
+        self.assertEqual(resp.status_code, 200)
+        self.assertContains(resp, self.site2.domain)
+        self.assertEqual(settings.SITE_ID, self.site2.pk)
+
+
+@pytest.mark.django_db
 @skipUnless(Site._meta.installed,
             'django.contrib.sites is not in settings.INSTALLED_APPS')
 @override_settings(
@@ -186,57 +231,26 @@ class DynamicSiteMiddlewareFallbackTest(TestCase):
 
         Site.objects.all().delete()
 
-        self.middleware = DynamicSiteMiddleware()
-
-    def tearDown(self):
-        settings.SITE_ID.reset()
-
     def test_404(self):
         request = self.factory.get('/')
         self.assertRaises(Http404,
-                          self.middleware.process_request, request)
+                          DynamicSiteMiddleware().process_request, request)
         self.assertEqual(settings.SITE_ID, 0)
 
     def test_testserver(self):
         host = 'testserver'
         site = Site.objects.create(domain=host)
         request = self.factory.get('/', host=host)
-        self.assertEqual(self.middleware.process_request(request), None)
+        self.assertEqual(DynamicSiteMiddleware().process_request(request), None)
         self.assertEqual(settings.SITE_ID, site.pk)
 
-    @skipIf(django.VERSION >= (1, 5, 0), "Starting Django 1.5, there are no "
-                                         "function based views.")
-    def test_string_function(self):
-        # Function based
-        settings.MULTISITE_FALLBACK = 'django.views.generic.simple.redirect_to'
-        settings.MULTISITE_FALLBACK_KWARGS = {'url': 'http://example.com/',
-                                              'permanent': False}
-        request = self.factory.get('/')
-        response = self.middleware.process_request(request)
-        self.assertEqual(response.status_code, 302)
-        self.assertEqual(response['Location'],
-                         settings.MULTISITE_FALLBACK_KWARGS['url'])
-
     def test_string_class(self):
         # Class based
         settings.MULTISITE_FALLBACK = 'django.views.generic.base.RedirectView'
         settings.MULTISITE_FALLBACK_KWARGS = {'url': 'http://example.com/',
                                               'permanent': False}
         request = self.factory.get('/')
-        response = self.middleware.process_request(request)
-        self.assertEqual(response.status_code, 302)
-        self.assertEqual(response['Location'],
-                         settings.MULTISITE_FALLBACK_KWARGS['url'])
-
-    @skipIf(django.VERSION >= (1, 5, 0), "Starting Django 1.5, there are no "
-                                         "function based views.")
-    def test_function_view(self):
-        from django.views.generic.simple import redirect_to
-        settings.MULTISITE_FALLBACK = redirect_to
-        settings.MULTISITE_FALLBACK_KWARGS = {'url': 'http://example.com/',
-                                              'permanent': False}
-        request = self.factory.get('/')
-        response = self.middleware.process_request(request)
+        response = DynamicSiteMiddleware().process_request(request)
         self.assertEqual(response.status_code, 302)
         self.assertEqual(response['Location'],
                          settings.MULTISITE_FALLBACK_KWARGS['url'])
@@ -247,7 +261,7 @@ class DynamicSiteMiddlewareFallbackTest(TestCase):
             url='http://example.com/', permanent=False
         )
         request = self.factory.get('/')
-        response = self.middleware.process_request(request)
+        response = DynamicSiteMiddleware().process_request(request)
         self.assertEqual(response.status_code, 302)
         self.assertEqual(response['Location'], 'http://example.com/')
 
@@ -255,9 +269,10 @@ class DynamicSiteMiddlewareFallbackTest(TestCase):
         settings.MULTISITE_FALLBACK = ''
         request = self.factory.get('/')
         self.assertRaises(ImproperlyConfigured,
-                          self.middleware.process_request, request)
+                          DynamicSiteMiddleware().process_request, request)
 
 
+@pytest.mark.django_db
 @skipUnless(Site._meta.installed,
             'django.contrib.sites is not in settings.INSTALLED_APPS')
 @override_settings(SITE_ID=0,)
@@ -266,6 +281,7 @@ class DynamicSiteMiddlewareSettingsTest(TestCase):
         self.assertRaises(TypeError, DynamicSiteMiddleware)
 
 
+@pytest.mark.django_db
 @override_settings(
     SITE_ID=SiteID(default=0),
     CACHE_MULTISITE_ALIAS='multisite',
@@ -273,6 +289,7 @@ class DynamicSiteMiddlewareSettingsTest(TestCase):
         'multisite': {'BACKEND': 'django.core.cache.backends.locmem.LocMemCache'}
     },
     MULTISITE_FALLBACK=None,
+    ALLOWED_HOSTS=ALLOWED_HOSTS
 )
 class CacheTest(TestCase):
     def setUp(self):
@@ -282,28 +299,28 @@ class CacheTest(TestCase):
         Site.objects.all().delete()
         self.site = Site.objects.create(domain=self.host)
 
-        self.middleware = DynamicSiteMiddleware()
-
     def test_site_domain_changed(self):
         # Test to ensure that the cache is cleared properly
-        cache_key = self.middleware.get_cache_key(self.host)
-        self.assertEqual(self.middleware.cache.get(cache_key), None)
+        middleware = DynamicSiteMiddleware()
+        cache_key = middleware.get_cache_key(self.host)
+        self.assertEqual(middleware.cache.get(cache_key), None)
         # Make the request
         request = self.factory.get('/')
-        self.assertEqual(self.middleware.process_request(request), None)
-        self.assertEqual(self.middleware.cache.get(cache_key).site_id,
+        self.assertEqual(middleware.process_request(request), None)
+        self.assertEqual(middleware.cache.get(cache_key).site_id,
                          self.site.pk)
         # Change the domain name
         self.site.domain = 'example.org'
         self.site.save()
-        self.assertEqual(self.middleware.cache.get(cache_key), None)
+        self.assertEqual(middleware.cache.get(cache_key), None)
         # Make the request again, which will now be invalid
         request = self.factory.get('/')
         self.assertRaises(Http404,
-                          self.middleware.process_request, request)
+                          middleware.process_request, request)
         self.assertEqual(settings.SITE_ID, 0)
 
 
+@pytest.mark.django_db
 @skipUnless(Site._meta.installed,
             'django.contrib.sites is not in settings.INSTALLED_APPS')
 @override_settings(SITE_ID=SiteID(),)
@@ -393,31 +410,31 @@ class SiteCacheTest(TestCase):
         )
 
     @override_settings(
-       CACHES={'default': {
+        CACHE_MULTISITE_ALIAS='multisite',
+        CACHES={
+            'multisite': {
                 'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
-                'KEY_PREFIX': 'test1'
-       }}
+                'KEY_PREFIX': 'looselycoupled'
+            }
+        },
     )
     def test_default_key_prefix(self):
+        """
+        If CACHE_MULTISITE_KEY_PREFIX is undefined,
+        the caching system should use CACHES[current]['KEY_PREFIX'].
+        """
         self._initialize_cache()
         # Populate cache
         self.assertEqual(Site.objects.get_current(), self.site)
         self.assertEqual(self.cache[self.site.id], self.site)
         self.assertEqual(
             self.cache._cache._get_cache_key(self.site.id),
-            'sites.{}.{}'.format(
-                settings.CACHES['default']['KEY_PREFIX'], self.site.id
-            ),
-            self.cache._cache._get_cache_key(self.site.id)
+            "sites.looselycoupled.{}".format(self.site.id)
         )
 
     @override_settings(
-       CACHE_MULTISITE_KEY_PREFIX="__test__",
-       CACHES={'default': {
-                'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
-                'KEY_PREFIX': 'test1'
-       }}
-    )
+        CACHE_MULTISITE_KEY_PREFIX="virtuouslyvirtual",
+        )
     def test_multisite_key_prefix_takes_priority_over_default(self):
         self._initialize_cache()
         # Populate cache
@@ -425,13 +442,11 @@ class SiteCacheTest(TestCase):
         self.assertEqual(self.cache[self.site.id], self.site)
         self.assertEqual(
             self.cache._cache._get_cache_key(self.site.id),
-            'sites.{}.{}'.format(
-                settings.CACHE_MULTISITE_KEY_PREFIX, self.site.id
-            ),
-            self.cache._cache._get_cache_key(self.site.id)
+            "sites.virtuouslyvirtual.{}".format(self.site.id)
         )
 
 
+@pytest.mark.django_db
 class TestSiteID(TestCase):
     def setUp(self):
         Site.objects.all().delete()
@@ -462,14 +477,12 @@ class TestSiteID(TestCase):
 
     def test_compare_differing_types(self):
         self.site_id.set(1)
-        # SiteIDHook <op> int
         self.assertNotEqual(self.site_id, '1')
         self.assertFalse(self.site_id == '1')
         self.assertTrue(self.site_id < '1')
         self.assertTrue(self.site_id <= '1')
         self.assertFalse(self.site_id > '1')
         self.assertFalse(self.site_id >= '1')
-        # int <op> SiteIDHook
         self.assertNotEqual('1', self.site_id)
         self.assertFalse('1' == self.site_id)
         self.assertFalse('1' < self.site_id)
@@ -506,6 +519,7 @@ class TestSiteID(TestCase):
         self.assertEqual(self.site_id.site_id, None)
 
 
+@pytest.mark.django_db
 @skipUnless(Site._meta.installed,
             'django.contrib.sites is not in settings.INSTALLED_APPS')
 class TestSiteDomain(TestCase):
@@ -518,8 +532,8 @@ class TestSiteDomain(TestCase):
         self.assertEqual(int(SiteDomain(default=self.domain)), self.site.id)
         self.assertRaises(Site.DoesNotExist,
                           int, SiteDomain(default='invalid'))
-        self.assertRaises(ValueError, SiteDomain, default=None)
-        self.assertRaises(ValueError, SiteDomain, default=1)
+        self.assertRaises(TypeError, SiteDomain, default=None)
+        self.assertRaises(TypeError, SiteDomain, default=1)
 
     def test_deferred_site(self):
         domain = 'example.org'
@@ -530,23 +544,7 @@ class TestSiteDomain(TestCase):
                          site.id)
 
 
-class TestSiteIDHook(TestCase):
-    def test_deprecation_warning(self):
-        with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter('always')
-            threadlocals.__warningregistry__ = {}
-            SiteIDHook()
-            self.assertTrue(w)
-            self.assertTrue(issubclass(w[-1].category, DeprecationWarning))
-
-    def test_default_value(self):
-        with warnings.catch_warnings():
-            warnings.simplefilter('ignore')
-            site_id = SiteIDHook()
-            self.assertEqual(site_id.default, 1)
-            self.assertEqual(int(site_id), 1)
-
-
+@pytest.mark.django_db
 class AliasTest(TestCase):
     def setUp(self):
         Alias.objects.all().delete()
@@ -780,18 +778,29 @@ class AliasTest(TestCase):
                          alias)
 
 
+
+@pytest.mark.django_db
 @override_settings(
     MULTISITE_COOKIE_DOMAIN_DEPTH=0,
     MULTISITE_PUBLIC_SUFFIX_LIST_CACHE=None,
+    ALLOWED_HOSTS=ALLOWED_HOSTS,
+    MULTISITE_EXTRA_HOSTS=['.extrahost.com']
 )
 class TestCookieDomainMiddleware(TestCase):
+
     def setUp(self):
         self.factory = RequestFactory(host='example.com')
-        self.middleware = CookieDomainMiddleware()
+        Site.objects.all().delete()
+        # create sites so we populate ALLOWED_HOSTS
+        Site.objects.create(domain='example.com')
+        Site.objects.create(domain='test.example.com')
+        Site.objects.create(domain='app.test1.example.com')
+        Site.objects.create(domain='app.test2.example.com')
+        Site.objects.create(domain='new.app.test3.example.com')
 
     def test_init(self):
-        self.assertEqual(self.middleware.depth, 0)
-        self.assertEqual(self.middleware.psl_cache,
+        self.assertEqual(CookieDomainMiddleware().depth, 0)
+        self.assertEqual(CookieDomainMiddleware().psl_cache,
                          os.path.join(tempfile.gettempdir(),
                                       'multisite_tld.dat'))
 
@@ -811,18 +820,26 @@ class TestCookieDomainMiddleware(TestCase):
         # No cookies
         request = self.factory.get('/')
         response = HttpResponse()
-        self.assertEqual(self.middleware.match_cookies(request, response),
+        self.assertEqual(CookieDomainMiddleware().match_cookies(request, response),
                          [])
-        cookies = self.middleware.process_response(request, response).cookies
-        self.assertEqual(cookies.values(), [])
+        cookies = CookieDomainMiddleware().process_response(request, response).cookies
+        self.assertEqual(list(cookies.values()), [])
 
         # Add some cookies with their domains already set
         response.set_cookie(key='a', value='a', domain='.example.org')
         response.set_cookie(key='b', value='b', domain='.example.co.uk')
-        self.assertEqual(self.middleware.match_cookies(request, response),
+        self.assertEqual(CookieDomainMiddleware().match_cookies(request, response),
                          [])
-        cookies = self.middleware.process_response(request, response).cookies
-        self.assertEqual(cookies.values(), [cookies['a'], cookies['b']])
+        cookies = CookieDomainMiddleware().process_response(request, response).cookies
+
+        if sys.version_info.major < 3:  # for testing under Python 2.X
+            self.assertItemsEqual(
+                list(cookies.values()), [cookies['a'], cookies['b']]
+            )
+        else:
+            self.assertCountEqual(
+                list(cookies.values()), [cookies['a'], cookies['b']]
+            )
         self.assertEqual(cookies['a']['domain'], '.example.org')
         self.assertEqual(cookies['b']['domain'], '.example.co.uk')
 
@@ -830,60 +847,78 @@ class TestCookieDomainMiddleware(TestCase):
         request = self.factory.get('/')
         response = HttpResponse()
         response.set_cookie(key='a', value='a', domain=None)
-        self.assertEqual(self.middleware.match_cookies(request, response),
+        self.assertEqual(CookieDomainMiddleware().match_cookies(request, response),
                          [response.cookies['a']])
         # No new cookies should be introduced
-        cookies = self.middleware.process_response(request, response).cookies
-        self.assertEqual(cookies.values(), [cookies['a']])
+        cookies = CookieDomainMiddleware().process_response(request, response).cookies
+        self.assertEqual(list(cookies.values()), [cookies['a']])
 
     def test_ip_address(self):
         response = HttpResponse()
         response.set_cookie(key='a', value='a', domain=None)
+        allowed = [host for host in ALLOWED_HOSTS] + ['192.0.43.10']
         # IP addresses should not be mutated
-        request = self.factory.get('/', host='192.0.43.10')
-        cookies = self.middleware.process_response(request, response).cookies
+        with override_settings(ALLOWED_HOSTS=allowed):
+            request = self.factory.get('/', host='192.0.43.10')
+            cookies = CookieDomainMiddleware().process_response(request, response).cookies
         self.assertEqual(cookies['a']['domain'], '')
 
     def test_localpath(self):
         response = HttpResponse()
         response.set_cookie(key='a', value='a', domain=None)
-        # Local domains should not be mutated
-        request = self.factory.get('/', host='localhost')
-        cookies = self.middleware.process_response(request, response).cookies
-        self.assertEqual(cookies['a']['domain'], '')
-        # Even local subdomains
-        request = self.factory.get('/', host='localhost.localdomain')
-        cookies = self.middleware.process_response(request, response).cookies
+
+        allowed = [host for host in ALLOWED_HOSTS] + \
+                  ['localhost', 'localhost.localdomain']
+        with override_settings(ALLOWED_HOSTS=allowed):
+            # Local domains should not be mutated
+            request = self.factory.get('/', host='localhost')
+            cookies = CookieDomainMiddleware().process_response(request, response).cookies
+            self.assertEqual(cookies['a']['domain'], '')
+            # Even local subdomains
+            request = self.factory.get('/', host='localhost.localdomain')
+            cookies = CookieDomainMiddleware().process_response(request, response).cookies
         self.assertEqual(cookies['a']['domain'], '')
 
     def test_simple_tld(self):
         response = HttpResponse()
         response.set_cookie(key='a', value='a', domain=None)
-        # Top-level domains shouldn't get mutated
-        request = self.factory.get('/', host='ai')
-        cookies = self.middleware.process_response(request, response).cookies
-        self.assertEqual(cookies['a']['domain'], '')
-        # Domains inside a TLD are OK
-        request = self.factory.get('/', host='www.ai')
-        cookies = self.middleware.process_response(request, response).cookies
+
+        allowed = [host for host in ALLOWED_HOSTS] + \
+                  ['ai', 'www.ai']
+        with override_settings(ALLOWED_HOSTS=allowed):
+            # Top-level domains shouldn't get mutated
+            request = self.factory.get('/', host='ai')
+            cookies = CookieDomainMiddleware().process_response(request, response).cookies
+            self.assertEqual(cookies['a']['domain'], '')
+            # Domains inside a TLD are OK
+            request = self.factory.get('/', host='www.ai')
+            cookies = CookieDomainMiddleware().process_response(request, response).cookies
         self.assertEqual(cookies['a']['domain'], '.www.ai')
 
     def test_effective_tld(self):
         response = HttpResponse()
         response.set_cookie(key='a', value='a', domain=None)
-        # Effective top-level domains with a webserver shouldn't get mutated
-        request = self.factory.get('/', host='com.ai')
-        cookies = self.middleware.process_response(request, response).cookies
-        self.assertEqual(cookies['a']['domain'], '')
-        # Domains within an effective TLD are OK
-        request = self.factory.get('/', host='nic.com.ai')
-        cookies = self.middleware.process_response(request, response).cookies
+
+        allowed = [host for host in ALLOWED_HOSTS] + \
+                  ['com.ai', 'nic.com.ai']
+        with override_settings(ALLOWED_HOSTS=allowed):
+            # Effective top-level domains with a webserver shouldn't get mutated
+            request = self.factory.get('/', host='com.ai')
+            cookies = CookieDomainMiddleware().process_response(request, response).cookies
+            self.assertEqual(cookies['a']['domain'], '')
+            # Domains within an effective TLD are OK
+            request = self.factory.get('/', host='nic.com.ai')
+            cookies = CookieDomainMiddleware().process_response(request, response).cookies
         self.assertEqual(cookies['a']['domain'], '.nic.com.ai')
 
     def test_subdomain_depth(self):
         response = HttpResponse()
         response.set_cookie(key='a', value='a', domain=None)
-        with override_settings(MULTISITE_COOKIE_DOMAIN_DEPTH=1):
+
+        allowed = [host for host in ALLOWED_HOSTS] + ['com']
+        with override_settings(
+                MULTISITE_COOKIE_DOMAIN_DEPTH=1, ALLOWED_HOSTS=allowed
+        ):
             # At depth 1:
             middleware = CookieDomainMiddleware()
             # Top-level domains are ignored
@@ -895,11 +930,169 @@ class TestCookieDomainMiddleware(TestCase):
             cookies = middleware.process_response(request, response).cookies
             self.assertEqual(cookies['a']['domain'], '')
             # But subdomains will get matched
-            request = self.factory.get('/', host='www.example.com')
+            request = self.factory.get('/', host='test.example.com')
             cookies = middleware.process_response(request, response).cookies
-            self.assertEqual(cookies['a']['domain'], '.www.example.com')
-            # And sub-subdomains will get matched
+            self.assertEqual(cookies['a']['domain'], '.test.example.com')
+            # And sub-subdomains will get matched to 1 level deep
             cookies['a']['domain'] = ''
-            request = self.factory.get('/', host='www.us.app.example.com')
+            request = self.factory.get('/', host='app.test1.example.com')
             cookies = middleware.process_response(request, response).cookies
-            self.assertEqual(cookies['a']['domain'], '.app.example.com')
+            self.assertEqual(cookies['a']['domain'], '.test1.example.com')
+
+    def test_subdomain_depth_2(self):
+        response = HttpResponse()
+        response.set_cookie(key='a', value='a', domain=None)
+        with override_settings(MULTISITE_COOKIE_DOMAIN_DEPTH=2):
+            # At MULTISITE_COOKIE_DOMAIN_DEPTH 2, subdomains are matched to
+            # 2 levels deep
+            middleware = CookieDomainMiddleware()
+            request = self.factory.get('/', host='app.test2.example.com')
+            cookies = middleware.process_response(request, response).cookies
+            self.assertEqual(cookies['a']['domain'], '.app.test2.example.com')
+            cookies['a']['domain'] = ''
+            request = self.factory.get('/', host='new.app.test3.example.com')
+            cookies = middleware.process_response(request, response).cookies
+            self.assertEqual(cookies['a']['domain'], '.app.test3.example.com')
+
+    def test_wildcard_subdomains(self):
+        response = HttpResponse()
+        response.set_cookie(key='a', value='a', domain=None)
+
+        allowed = [host for host in ALLOWED_HOSTS] + ['.test.example.com']
+        with override_settings(
+                MULTISITE_COOKIE_DOMAIN_DEPTH=2, ALLOWED_HOSTS=allowed
+        ):
+            # At MULTISITE_COOKIE_DOMAIN_DEPTH 2, subdomains are matched to
+            # 2 levels deep against the wildcard
+            middleware = CookieDomainMiddleware()
+            request = self.factory.get('/', host='foo.test.example.com')
+            cookies = middleware.process_response(request, response).cookies
+            self.assertEqual(cookies['a']['domain'], '.foo.test.example.com')
+            cookies['a']['domain'] = ''
+            request = self.factory.get('/', host='foo.bar.test.example.com')
+            cookies = middleware.process_response(request, response).cookies
+            self.assertEqual(cookies['a']['domain'], '.bar.test.example.com')
+
+    def test_multisite_extra_hosts(self):
+        # MULTISITE_EXTRA_HOSTS is set to ['.extrahost.com'] but
+        # ALLOWED_HOSTS seems to be genereated in override_settings before
+        # the extra hosts is added, so we need to recalculate it here.
+        allowed = IterableLazyObject(lambda: AllowedHosts())
+        with override_settings(ALLOWED_HOSTS=allowed):
+            response = HttpResponse()
+            response.set_cookie(key='a', value='a', domain=None)
+            middleware = CookieDomainMiddleware()
+            request = self.factory.get('/', host='test.extrahost.com')
+            cookies = middleware.process_response(request, response).cookies
+            self.assertEqual(cookies['a']['domain'], '.extrahost.com')
+            cookies['a']['domain'] = ''
+            request = self.factory.get('/', host='foo.extrahost.com')
+            cookies = middleware.process_response(request, response).cookies
+            self.assertEqual(cookies['a']['domain'], '.extrahost.com')
+            cookies['a']['domain'] = ''
+            request = self.factory.get('/', host='foo.bar.extrahost.com')
+            cookies = middleware.process_response(request, response).cookies
+            self.assertEqual(cookies['a']['domain'], '.extrahost.com')
+
+
+if django.VERSION < (1, 8):
+    TEMPLATE_SETTINGS = {
+        'TEMPLATE_LOADERS': ['multisite.template.loaders.filesystem.Loader'],
+        'TEMPLATE_DIRS': [os.path.join(os.path.abspath(os.path.dirname(__file__)),
+                                  'test_templates')]
+    }
+else:
+    TEMPLATE_SETTINGS = {'TEMPLATES':[
+        {
+            'BACKEND': 'django.template.backends.django.DjangoTemplates',
+            'DIRS': [
+                os.path.join(os.path.abspath(os.path.dirname(__file__)),
+                             'test_templates')
+            ],
+            'OPTIONS': {
+                'loaders': [
+                    'multisite.template.loaders.filesystem.Loader',
+                ]
+            },
+        }
+    ]
+    }
+
+
+@override_settings(
+    MULTISITE_DEFAULT_TEMPLATE_DIR='multisite_templates',
+    **TEMPLATE_SETTINGS
+)
+class TemplateLoaderTests(TestCase):
+
+    def test_get_template_multisite_default_dir(self):
+        template = get_template("test.html")
+        self.assertEqual(template.render(), "Test!")
+
+    def test_domain_template(self):
+        template = get_template("example.html")
+        self.assertEqual(template.render(), "Test example.com template")
+
+    def test_get_template_old_settings(self):
+        # tests that we can still get to the template filesystem loader with
+        # the old setting configuration
+        with override_settings(
+                TEMPLATES=[
+                    {
+                        'BACKEND': 'django.template.backends.django.DjangoTemplates',
+                        'DIRS': [
+                            os.path.join(
+                                os.path.abspath(os.path.dirname(__file__)),
+                                'test_templates')
+                        ],
+                        'OPTIONS': {
+                            'loaders': [
+                                'multisite.template_loader.Loader',
+                            ]
+                        },
+                    }
+                ]
+        ):
+            template = get_template("test.html")
+            self.assertEqual(template.render(), "Test!")
+
+
+class UpdatePublicSuffixListCommandTestCase(TestCase):
+
+    def setUp(self):
+        self.cache_file = '/tmp/multisite_tld.dat'
+
+        # save the tldextract logger output to a buffer to test output
+        self.out = StringIO()
+        self.logger = logging.getLogger('tldextract')
+        self.logger.setLevel(logging.DEBUG)
+        stdout_handler = logging.StreamHandler(self.out)
+        stdout_handler.setLevel(logging.DEBUG)
+        self.logger.addHandler(stdout_handler)
+
+        # patch tldextract to avoid actual requests
+        self.patcher = mock.patch('tldextract.TLDExtract')
+        self.tldextract = self.patcher.start()
+
+    def tearDown(self):
+        self.patcher.stop()
+
+    def tldextract_update_side_effect(self, *args, **kwargs):
+        self.logger.debug('TLDExtract.update called')
+
+    def test_command(self):
+        call_command('update_public_suffix_list')
+        expected_calls = [
+            mock.call(cache_file=self.cache_file),
+            mock.call().update(fetch_now=True)
+        ]
+        self.assertEqual(self.tldextract.mock_calls, expected_calls)
+
+    def test_command_output(self):
+        # make sure that the logger receives output from the method
+        self.tldextract().update.side_effect = self.tldextract_update_side_effect
+
+        call_command('update_public_suffix_list', verbosity=3)
+        update_message = 'Updating {}'.format(self.cache_file)
+        self.assertIn(update_message, self.out.getvalue())
+        self.assertIn('TLDExtract.update called', self.out.getvalue())
